"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/index.ts
var src_exports = {};
__export(src_exports, {
  FunctionHandlerError: () => FunctionHandlerError,
  FunctionReducerError: () => FunctionReducerError,
  assertSingleToolCall: () => assertSingleToolCall,
  createFunctionHandler: () => createFunctionHandler,
  handleToolCalls: () => handleToolCalls,
  parseArguments: () => parseArguments,
  reduceToolCalls: () => reduceToolCalls,
  toJsonSchema: () => toJsonSchema,
  toTool: () => toTool
});
module.exports = __toCommonJS(src_exports);

// src/function.ts
var import_zod_to_json_schema = __toESM(require("zod-to-json-schema"));
function toJsonSchema(functionDef) {
  const { name, description, schema } = functionDef;
  const jsonSchema = (0, import_zod_to_json_schema.default)(schema);
  const { type, properties, required } = jsonSchema;
  return {
    name,
    description,
    parameters: {
      type,
      properties,
      required
    }
  };
}
function toTool(functionDef) {
  return {
    type: "function",
    function: toJsonSchema(functionDef)
  };
}

// src/utils/errors.ts
var HttpError = class extends Error {
  constructor(message, status = 500, source) {
    super(message);
    this.status = status;
    this.source = source;
    this.name = this.constructor.name;
  }
};

// src/utils/logging.ts
var defaultLogger = {
  info: process.env.NODE_ENV === "development" ? console.info : () => {
  },
  log: process.env.NODE_ENV === "development" ? console.log : () => {
  },
  error: console.error
};
var currentLogger = defaultLogger;
var logger = {
  info: (...args) => currentLogger.info(...args),
  log: (...args) => currentLogger.log(...args),
  error: (...args) => currentLogger.error(...args)
};

// src/utils/parse.ts
var import_zod_validation_error = require("zod-validation-error");
var InvalidFunctionName = class extends HttpError {
  constructor(name) {
    const message = `Function not found: ${name}`;
    super(message, 500);
  }
};
var InvalidFunctionArguments = class extends HttpError {
  constructor(name, args, zodError) {
    const validationError = (0, import_zod_validation_error.fromZodError)(zodError).message;
    const message = `Invalid ${name} function args: ${validationError}`;
    const source = "OpenAI";
    logger.error(`${message} (${args})`);
    super(message, 500, source);
  }
};
function findFunction(name, functions) {
  const func = functions.find((f) => f.name === name);
  if (!func) {
    throw new InvalidFunctionName(name);
  }
  return func;
}
function parseArguments(name, args, schema) {
  const parameters = JSON.parse(args);
  const result = schema.safeParse(parameters);
  if (!result.success) {
    throw new InvalidFunctionArguments(name, args, result.error);
  }
  return result.data;
}

// src/handler.ts
var FunctionHandlerError = class extends HttpError {
  constructor(name, err) {
    const message = err instanceof Error ? err.message : "Unknown error";
    super(`Error calling function ${name}: ${message}`, 500);
  }
};
function createFunctionHandler({
  name,
  description,
  schema,
  handler
}) {
  return { name, description, schema, handler };
}
function findHandler(name, handlers) {
  const handler = handlers.find((h) => h.name === name);
  if (!handler) {
    throw new InvalidFunctionName(name);
  }
  return handler;
}
function handleSingleToolCall(handlers, toolCall) {
  return __async(this, null, function* () {
    const { name } = toolCall.function;
    const handler = findHandler(name, handlers);
    const parameters = parseArguments(
      toolCall.function.name,
      toolCall.function.arguments,
      handler.schema
    );
    try {
      logger.info("Calling function: ", name);
      const output = yield handler.handler(parameters);
      logger.info("Tool handler output: ", output);
      return output;
    } catch (err) {
      throw new FunctionHandlerError(name, err);
    }
  });
}
function handleToolCalls(handlers, toolCalls) {
  return __async(this, null, function* () {
    if (toolCalls === void 0) {
      return [];
    }
    const toolOutputs = yield Promise.all(
      toolCalls.map((toolCall) => handleSingleToolCall(handlers, toolCall))
    );
    return toolOutputs;
  });
}

// src/reducer.ts
var FunctionReducerError = class extends HttpError {
  constructor(name, err) {
    const message = err instanceof Error ? err.message : "Unknown error";
    super(`Error calling reducer ${name}: ${message}`, 500);
  }
};
function reduceToolCalls(prevState, functionReducer, toolCalls) {
  return __async(this, null, function* () {
    if (toolCalls === void 0) {
      return prevState;
    }
    let state = prevState;
    toolCalls.forEach((toolCall) => {
      const { name } = toolCall.function;
      const func = findFunction(name, functionReducer.functions);
      const parameters = parseArguments(name, toolCall.function.arguments, func.schema);
      try {
        logger.info("Calling reducer: ", name);
        state = functionReducer.reducer(state, name, parameters);
        logger.info("New state: ", state);
      } catch (err) {
        throw new FunctionReducerError(name, err);
      }
    });
    return state;
  });
}

// src/utils/assert.ts
function assertSingleToolCall(completion, label) {
  const prefix = label ? `Unexpected (${label})` : "Unexpected";
  if (completion.choices.length !== 1) {
    throw new Error(`${prefix}: got ${completion.choices.length} choices`);
  }
  const { tool_calls } = completion.choices[0].message;
  if (tool_calls === void 0) {
    throw new Error(`${prefix}: no tool_calls for ${label}`);
  }
  if (tool_calls.length !== 1) {
    logger.error(
      `${prefix}: Multiple tool calls for ${label}: ${JSON.stringify(
        tool_calls,
        null,
        2
      )}`
    );
    throw new Error(`${prefix}: got ${tool_calls.length} tool_calls`);
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  FunctionHandlerError,
  FunctionReducerError,
  assertSingleToolCall,
  createFunctionHandler,
  handleToolCalls,
  parseArguments,
  reduceToolCalls,
  toJsonSchema,
  toTool
});
//# sourceMappingURL=index.js.map